/******************************************************
* Zeus CMD - OpenGL ES Tutorial 14 : Backface Culling *
* By Grant James (ZEUS)                               *
* http://www.zeuscmd.com                              *
******************************************************/
#pragma comment(lib, "libGLES_CM.lib")
#pragma comment(lib, "ug.lib")

#include "ug.h"

float xrot = 0.0f;
float yrot = 0.0f;

bool acw = true;

GLfloat box[] = {
	// FRONT
	-0.5f, -0.5f,  0.5f,
	 0.5f, -0.5f,  0.5f,
	-0.5f,  0.5f,  0.5f,
	 0.5f,  0.5f,  0.5f,
	// BACK
	-0.5f, -0.5f, -0.5f,
	-0.5f,  0.5f, -0.5f,
	 0.5f, -0.5f, -0.5f,
	 0.5f,  0.5f, -0.5f,
	// LEFT
	-0.5f, -0.5f,  0.5f,
	-0.5f,  0.5f,  0.5f,
	-0.5f, -0.5f, -0.5f,
	-0.5f,  0.5f, -0.5f,
	// RIGHT
	 0.5f, -0.5f, -0.5f,
	 0.5f,  0.5f, -0.5f,
	 0.5f, -0.5f,  0.5f,
	 0.5f,  0.5f,  0.5f,
	// BOTTOM
	-0.5f, -0.5f,  0.5f,
	-0.5f, -0.5f, -0.5f,
	 0.5f, -0.5f,  0.5f,
	 0.5f, -0.5f, -0.5f,
};

void init()
{
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);

	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	glClearDepthf(1.0f);

	glVertexPointer(3, GL_FLOAT, 0, box);
	glEnableClientState(GL_VERTEX_ARRAY);

	glShadeModel(GL_FLAT);

	glEnable(GL_CULL_FACE);
}

void display(UGWindow uwin)
{
   glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   glLoadIdentity();

   ugluLookAtf(
		0.0f, 0.0f, 3.0f,
		0.0f, 0.0f, 0.0f,
		0.0f, 1.0f, 0.0f);

   glRotatef(xrot, 1.0f, 0.0f, 0.0f);
   glRotatef(yrot, 0.0f, 1.0f, 0.0f);

   glColor4f(1.0f, 0.0f, 0.0f, 1.0f);
   glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
   glDrawArrays(GL_TRIANGLE_STRIP, 4, 4);

   glColor4f(0.0f, 1.0f, 0.0f, 1.0f);
   glDrawArrays(GL_TRIANGLE_STRIP, 8, 4);
   glDrawArrays(GL_TRIANGLE_STRIP, 12, 4);

   glColor4f(0.0f, 0.0f, 1.0f, 1.0f);
   glDrawArrays(GL_TRIANGLE_STRIP, 16, 4);

   glFlush ();
   ugSwapBuffers(uwin);
}

void idle(UGWindow uwin)
{
	xrot += 2.0f;
	yrot += 3.0f;

	ugPostRedisplay(uwin);
}

void keyboard(UGWindow uwin, int key, int x, int y)
{
	switch(key)
	{
	case 'q' : exit(0); break;
	case UG_KEY_UP : exit(0); break;
	case 'f' :
		if (glIsEnabled(GL_CULL_FACE))
			glDisable(GL_CULL_FACE);
		else
			glEnable(GL_CULL_FACE);
		break;
	case 'r' :
		acw = !acw;

		glFrontFace(acw ? GL_CCW : GL_CW);
		
		break;
	}
}

void reshape(UGWindow uwin, int width, int height)
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	glViewport(0, 0, width, height);
	ugluPerspectivef(45.0f, 1.0f * width / height, 1.0f, 100.0f);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

void pointer(UGWindow uwin, int button, int state, int x, int y)
{
	exit(0);
}

int main()
{
	UGCtx ug = ugInit();

	UGWindow uwin = ugCreateWindow(ug, "UG_DEPTH", "14 - Backface Culling", 250, 250, 100, 100);

	init();

	ugDisplayFunc(uwin, display);
	ugReshapeFunc(uwin, reshape);

	ugKeyboardFunc(uwin, keyboard);
	ugPointerFunc(uwin, pointer);

	ugIdleFunc(ug, idle);
	ugMainLoop(ug);

	return 0;
}
